@@ -21,10 +21,20 @@ fi
 
 tscriptname=`basename $0 .sh`
 export PKG=`echo $tscriptname | sed -e 's/\-[^\-]*\-[^\-]*$//'`
-export VER=`echo $tscriptname | sed -e 's/^[^\-]*\-//' -e 's/\-[^\-]*$//'`
-export REL=`echo $tscriptname | sed -e 's/^[^\-]*\-[^\-]*\-//'`
+export VER=`echo $tscriptname | sed -e "s/${PKG}\-//" -e 's/\-[^\-]*$//'`
+export REL=`echo $tscriptname | sed -e "s/${PKG}\-${VER}\-//"`
 export FULLPKG=${PKG}-${VER}-${REL}
-export src_orig_pkg_name=${PKG}-${VER}.tar.gz
+
+# determine correct decompression option and tarball filename
+if [ -e ${PKG}-${VER}.tar.gz ] ; then
+  export opt_decomp=z
+  export src_orig_pkg_ext=gz
+elif [ -e ${PKG}-${VER}.tar.bz2 ] ; then
+  export opt_decomp=j
+  export src_orig_pkg_ext=bz2
+fi
+
+export src_orig_pkg_name=${PKG}-${VER}.tar.${src_orig_pkg_ext}
 export src_pkg_name=${FULLPKG}-src.tar.bz2
 export src_patch_name=${FULLPKG}.patch
 export bin_pkg_name=${FULLPKG}.tar.bz2
@@ -48,98 +58,111 @@ CFLAGS="-O2"
 
 mkdirs() {
   (cd ${topdir} && \
+  rm -fr ${objdir} ${instdir} ${srcinstdir} && \
   mkdir -p ${objdir} && \
   mkdir -p ${instdir} && \
-  mkdir -p ${srcinstdir} )
-}
+  mkdir -p ${srcinstdir}
+)}
+
 prep() {
   (cd ${topdir} && \
-  tar xvzf ${src_orig_pkg} && \
+  tar xv${opt_decomp}f ${src_orig_pkg} && \
   mv ${PKG} ${PKG}-${VER}
   cd ${topdir} && \
   patch -p0 < ${src_patch} 
   && mkdirs && \
   chmod +x ${srcdir}/CYGWIN-PATCHES/shtool && \
   cd ${srcdir} && \
-  CYGWIN-PATCHES/shtool mkshadow -v . .build )
-}
+  CYGWIN-PATCHES/shtool mkshadow -v . .build
+)}
+
 conf() {
   echo Configure successful.
 }
+
 build() {
-  (cd ${objdir} && make )
-}
+  (cd ${objdir} && make
+)}
+
 check() {
   (cd ${objdir} && \
-  make test | tee ${checkfile} 2>&1 )
-}
+  make test | tee ${checkfile} 2>&1
+)}
+
 clean() {
   (cd ${objdir} && \
-  make clean )
-}
+  make clean
+)}
+
 install() {
   (cd ${objdir} && \
   make install DESTDIR=${instdir} && \
-   if [ -f ${instdir}${prefix}/info/dir ] ; then \
-    rm ${instdir}${prefix}/info/dir ; \
+   if [ -f ${instdir}${prefix}/share/info/dir ] ; then \
+    rm ${instdir}${prefix}/share/info/dir ; \
   fi && \
-  if [ ! -d ${instdir}${prefix}/doc/${PKG}-${VER} ]; then \
-    mkdir -p ${instdir}${prefix}/doc/${PKG}-${VER} ; \
+  if [ ! -d ${instdir}${prefix}/share/doc/${PKG}-${VER} ]; then \
+    mkdir -p ${instdir}${prefix}/share/doc/${PKG}-${VER} ; \
   fi && \
-  if [ ! -d ${instdir}${prefix}/doc/Cygwin ]; then \
-    mkdir -p ${instdir}${prefix}/doc/Cygwin ; \
+  if [ ! -d ${instdir}${prefix}/share/doc/Cygwin ]; then \
+    mkdir -p ${instdir}${prefix}/share/doc/Cygwin ; \
   fi && \
   templist=""; \
   for f in ${srcdir}/ANNOUNCE ${srcdir}/COPYING \
            ${srcdir}/INSTALL ${srcdir}/TODO ${srcdir}/libjbig/jbig.doc \
-			  ${srcdir}/pbmtools/*.txt ; do \
+           ${srcdir}/pbmtools/*.txt ; do \
     if [ -f $f ] ; then \
       templist="$templist $f"; \
     fi ; \
   done && \
   if [ ! "x$templist" = "x" ]; then \
     /usr/bin/install -m 644 $templist \
-         ${instdir}${prefix}/doc/${PKG}-${VER} ;
+         ${instdir}${prefix}/share/doc/${PKG}-${VER} ;
   fi && \
   if [ -f ${srcdir}/CYGWIN-PATCHES/${PKG}.README ]; then \
     /usr/bin/install -m 644 ${srcdir}/CYGWIN-PATCHES/${PKG}.README \
-      ${instdir}${prefix}/doc/Cygwin/${PKG}-${VER}.README ; \
+      ${instdir}${prefix}/share/doc/Cygwin/${PKG}-${VER}.README ; \
   else \
     if [ -f ${srcdir}/CYGWIN-PATCHES/README ]; then \
       /usr/bin/install -m 644 ${srcdir}/CYGWIN-PATCHES/README \
-        ${instdir}${prefix}/doc/Cygwin/${PKG}-${VER}.README ; \
+        ${instdir}${prefix}/share/doc/Cygwin/${PKG}-${VER}.README ; \
     fi ;\
-  fi )
-}
+  fi
+)}
+
 strip() {
   (cd ${instdir} && \
-  find . -name "*.exe" | xargs strip ;
-  find . -name "*.dll" | xargs strip )
-}
+  find . -name "*.exe" | xargs strip >/dev/null 2>&1
+  find . -name "*.dll" | xargs strip >/dev/null 2>&1
+)}
+
 pkg() {
   (cd ${instdir} && \
-  tar cvjf ${bin_pkg} * )
-}
+  tar cvjf ${bin_pkg} *
+)}
+
 mkpatch() {
   (cd ${srcdir} && \
-  tar xvzf ${src_orig_pkg} && \
+  tar xv${opt_decomp}f ${src_orig_pkg} && \
   mv ${PKG} ../${PKG}-${VER}-orig && \
   cd ${topdir} && \
   diff -urN -x '.build' -x '.inst' -x '.sinst' \
     ${PKG}-${VER}-orig ${PKG}-${VER} > \
     ${srcinstdir}/${src_patch_name} ; \
-  rm -rf ${PKG}-${VER}-orig )
-}
+  rm -rf ${PKG}-${VER}-orig
+)}
+
 spkg() {
   (mkpatch && \
   cp ${src_orig_pkg} ${srcinstdir}/${src_orig_pkg_name} && \
   cp $0 ${srcinstdir}/`basename $0` && \
   cd ${srcinstdir} && \
-  tar cvjf ${src_pkg} * )
-}
+  tar cvjf ${src_pkg} *
+)}
+
 finish() {
   rm -rf ${srcdir} 
 }
+
 case $1 in
   prep)	prep ; STATUS=$? ;;
   mkdirs)	mkdirs; STATUS=$? ;;
